@@ -11,10 +11,11 @@ import rasa.cli.utils
 import rasa.core.utils
 import rasa.utils.common
 from rasa.cli import SubParsersAction
-from rasa.core import ContextualResponseRephraser
+from rasa.core.nlg.contextual_response_rephraser import ContextualResponseRephraser
 from rasa.core.nlg.generator import NaturalLanguageGenerator
 from rasa.core.train import do_compare_training
 from rasa.engine.validation import validate_api_type_config_key_usage
+from rasa.exceptions import ValidationError
 from rasa.shared.constants import (
     CONFIG_MANDATORY_KEYS,
     CONFIG_MANDATORY_KEYS_CORE,
@@ -81,6 +82,13 @@ def _check_nlg_endpoint_validity(endpoint: Union[Path, str]) -> None:
                 ContextualResponseRephraser.__name__,
             )
         NaturalLanguageGenerator.create(endpoints.nlg)
+    except ValidationError as e:
+        structlogger.error(
+            e.code,
+            event_info=e.info,
+            **e.ctx,
+        )
+        sys.exit(1)
     except Exception as e:
         structlogger.error(
             "cli.train.nlg_failed_to_initialise.validation_error",
@@ -112,6 +120,10 @@ def run_training(args: argparse.Namespace, can_exit: bool = False) -> Optional[T
     )
     config = rasa.cli.utils.get_validated_config(args.config, CONFIG_MANDATORY_KEYS)
 
+    # Validates and loads endpoints with proper endpoint file location
+    # This will initialise the endpoints singleton properly so that
+    # it can be used safely throughout the codebase with
+    # `AvailableEndpoints.get_instance()`
     _check_nlg_endpoint_validity(args.endpoints)
 
     training_files = [
