@@ -31,9 +31,11 @@ from rasa.core.channels.voice_stream.tts.azure import AzureTTS
 from rasa.core.channels.voice_stream.tts.cartesia import CartesiaTTS
 from rasa.core.channels.voice_stream.tts.tts_cache import TTSCache
 from rasa.core.channels.voice_stream.tts.tts_engine import TTSEngine, TTSError
-from rasa.core.channels.voice_stream.util import generate_silence
-from rasa.shared.core.constants import SLOT_SILENCE_TIMEOUT
-from rasa.shared.utils.cli import print_error_and_exit
+from rasa.core.channels.voice_stream.util import (
+    generate_silence,
+)
+from rasa.shared.core.constants import SILENCE_TIMEOUT_SLOT
+from rasa.shared.exceptions import InvalidConfigException
 from rasa.shared.utils.common import (
     class_from_module_path,
     mark_as_beta_feature,
@@ -42,6 +44,11 @@ from rasa.utils.io import remove_emojis
 
 logger = structlog.get_logger(__name__)
 
+# define constants for the voice channel
+USER_CONVERSATION_SESSION_END = "/session_end"
+USER_CONVERSATION_SESSION_START = "/session_start"
+USER_CONVERSATION_SILENCE_TIMEOUT = "/silence_timeout"
+
 
 @dataclass
 class VoiceChannelAction:
@@ -64,6 +71,14 @@ class ContinueConversationAction(VoiceChannelAction):
 
 
 def asr_engine_from_config(asr_config: Dict) -> ASREngine:
+    if not asr_config:
+        raise ValueError("ASR configuration dictionary cannot be empty")
+
+    if "name" not in asr_config:
+        raise ValueError(
+            "ASR configuration must contain 'name' key specifying the engine type"
+        )
+
     name = str(asr_config["name"])
     asr_config = copy.copy(asr_config)
     asr_config.pop("name")
@@ -77,12 +92,12 @@ def asr_engine_from_config(asr_config: Dict) -> ASREngine:
             asr_engine_class = class_from_module_path(name)
             return asr_engine_class.from_config_dict(asr_config)
         except NameError:
-            print_error_and_exit(
+            raise InvalidConfigException(
                 f"Failed to initialize ASR Engine with type '{name}'. "
                 f"Please make sure the method `from_config_dict`is implemented."
             )
         except TypeError as e:
-            print_error_and_exit(
+            raise InvalidConfigException(
                 f"Failed to initialize ASR Engine with type '{name}'. "
                 f"Invalid configuration provided. "
                 f"Error: {e}"
@@ -90,6 +105,14 @@ def asr_engine_from_config(asr_config: Dict) -> ASREngine:
 
 
 def tts_engine_from_config(tts_config: Dict) -> TTSEngine:
+    if not tts_config:
+        raise ValueError("TTS configuration dictionary cannot be empty")
+
+    if "name" not in tts_config:
+        raise ValueError(
+            "TTS configuration must contain 'name' key specifying the engine type"
+        )
+
     name = str(tts_config["name"])
     tts_config = copy.copy(tts_config)
     tts_config.pop("name")
@@ -103,13 +126,13 @@ def tts_engine_from_config(tts_config: Dict) -> TTSEngine:
             tts_engine_class = class_from_module_path(name)
             return tts_engine_class.from_config_dict(tts_config)
         except NameError:
-            print_error_and_exit(
+            raise InvalidConfigException(
                 f"Failed to initialize TTS Engine with type '{name}'. "
                 f"Please make sure the method `from_config_dict`is implemented."
             )
         except TypeError as e:
-            print_error_and_exit(
-                f"Failed to initialize ASR Engine with type '{name}'. "
+            raise InvalidConfigException(
+                f"Failed to initialize TTS Engine with type '{name}'. "
                 f"Invalid configuration provided. "
                 f"Error: {e}"
             )
@@ -153,8 +176,6 @@ class VoiceOutputChannel(OutputChannel):
     async def send_start_marker(self, recipient_id: str) -> None:
         """Send a marker message before the first audio chunk."""
         # Default implementation uses the generic marker message
-        call_state.is_bot_speaking = True  # type: ignore[attr-defined]
-        VoiceInputChannel._cancel_silence_timeout_watcher()
         await self.send_marker_message(recipient_id)
 
     async def send_intermediate_marker(self, recipient_id: str) -> None:
@@ -168,8 +189,12 @@ class VoiceOutputChannel(OutputChannel):
     def update_silence_timeout(self) -> None:
         """Updates the silence timeout for the session."""
         if self.tracker_state:
-            call_state.silence_timeout = (  # type: ignore[attr-defined]
-                self.tracker_state["slots"][SLOT_SILENCE_TIMEOUT]
+            call_state.silence_timeout = self.tracker_state["slots"][  # type: ignore[attr-defined]
+                SILENCE_TIMEOUT_SLOT
+            ]
+            logger.debug(
+                "voice_channel.silence_timeout_updated",
+                silence_timeout=call_state.silence_timeout,
             )
 
     async def send_text_with_buttons(
@@ -191,6 +216,7 @@ class VoiceOutputChannel(OutputChannel):
         collected_audio_bytes = RasaAudioBytes(b"")
         seconds_marker = -1
         last_sent_offset = 0
+        logger.debug("voice_channel.sending_audio", text=text)
 
         # Send start marker before first chunk
         try:
@@ -276,18 +302,21 @@ class VoiceOutputChannel(OutputChannel):
 
 
 class VoiceInputChannel(InputChannel):
+    # All children of this class require a voice license to be used.
+    requires_voice_license = True
+
     def __init__(
         self,
         server_url: str,
         asr_config: Dict,
         tts_config: Dict,
-        monitor_silence: bool = False,
     ):
-        validate_voice_license_scope()
+        if self.requires_voice_license:
+            validate_voice_license_scope()
+
         self.server_url = server_url
         self.asr_config = asr_config
         self.tts_config = tts_config
-        self.monitor_silence = monitor_silence
         self.tts_cache = TTSCache(tts_config.get("cache_size", 1000))
 
         logger.info(
@@ -305,8 +334,6 @@ class VoiceInputChannel(InputChannel):
         timeout = call_state.silence_timeout
         if not timeout:
             return
-        if not self.monitor_silence:
-            return
         logger.debug("voice_channel.silence_timeout_watch_started", timeout=timeout)
         await asyncio.sleep(timeout)
         await asr_event_queue.put(UserSilence())
@@ -321,13 +348,28 @@ class VoiceInputChannel(InputChannel):
             call_state.silence_timeout_watcher = None  # type: ignore[attr-defined]
 
     @classmethod
-    def from_credentials(cls, credentials: Optional[Dict[str, Any]]) -> InputChannel:
-        credentials = credentials or {}
+    def from_credentials(
+        cls,
+        credentials: Optional[Dict[str, Any]],
+    ) -> InputChannel:
+        if not credentials:
+            cls.raise_missing_credentials_exception()
+
+        if not credentials.get("server_url"):
+            raise InvalidConfigException("No server_url provided in credentials.")
+        if not credentials.get("asr"):
+            raise InvalidConfigException(
+                "No ASR configuration provided in credentials."
+            )
+        if not credentials.get("tts"):
+            raise InvalidConfigException(
+                "No TTS configuration provided in credentials."
+            )
+
         return cls(
-            credentials["server_url"],
-            credentials["asr"],
-            credentials["tts"],
-            credentials.get("monitor_silence", False),
+            server_url=credentials["server_url"],
+            asr_config=credentials["asr"],
+            tts_config=credentials["tts"],
         )
 
     def channel_bytes_to_rasa_audio_bytes(self, input_bytes: bytes) -> RasaAudioBytes:
@@ -347,7 +389,7 @@ class VoiceInputChannel(InputChannel):
     ) -> None:
         output_channel = self.create_output_channel(channel_websocket, tts_engine)
         message = UserMessage(
-            text="/session_start",
+            text=USER_CONVERSATION_SESSION_START,
             output_channel=output_channel,
             sender_id=self.get_sender_id(call_parameters),
             input_channel=self.name(),
@@ -384,17 +426,17 @@ class VoiceInputChannel(InputChannel):
 
         async def consume_audio_bytes() -> None:
             async for message in channel_websocket:
-                is_bot_speaking_before = call_state.is_bot_speaking
+                was_bot_speaking_before = call_state.is_bot_speaking
                 channel_action = self.map_input_message(message, channel_websocket)
                 is_bot_speaking_after = call_state.is_bot_speaking
 
-                if not is_bot_speaking_before and is_bot_speaking_after:
+                if not was_bot_speaking_before and is_bot_speaking_after:
                     logger.debug("voice_channel.bot_started_speaking")
                     # relevant when the bot speaks multiple messages in one turn
                     self._cancel_silence_timeout_watcher()
 
                 # we just stopped speaking, starting a watcher for silence timeout
-                if is_bot_speaking_before and not is_bot_speaking_after:
+                if was_bot_speaking_before and not is_bot_speaking_after:
                     logger.debug("voice_channel.bot_stopped_speaking")
                     self._cancel_silence_timeout_watcher()
                     call_state.silence_timeout_watcher = (  # type: ignore[attr-defined]
@@ -485,7 +527,7 @@ class VoiceInputChannel(InputChannel):
         elif isinstance(e, UserSilence):
             output_channel = self.create_output_channel(voice_websocket, tts_engine)
             message = UserMessage(
-                text="/silence_timeout",
+                text=USER_CONVERSATION_SILENCE_TIMEOUT,
                 output_channel=output_channel,
                 sender_id=self.get_sender_id(call_parameters),
                 input_channel=self.name(),
@@ -503,7 +545,7 @@ class VoiceInputChannel(InputChannel):
         """Handle disconnection from the channel."""
         output_channel = self.create_output_channel(channel_websocket, tts_engine)
         message = UserMessage(
-            text="/session_end",
+            text=USER_CONVERSATION_SESSION_END,
             output_channel=output_channel,
             sender_id=self.get_sender_id(call_parameters),
             input_channel=self.name(),
