@@ -17,12 +17,14 @@ from rasa.shared.constants import (
     MODEL_NAME_CONFIG_KEY,
     OPENAI_PROVIDER,
     PROMPT_CONFIG_KEY,
+    PROMPT_TEMPLATE_CONFIG_KEY,
     PROVIDER_CONFIG_KEY,
     TEMPERATURE_CONFIG_KEY,
     TIMEOUT_CONFIG_KEY,
 )
 from rasa.shared.core.domain import KEY_RESPONSES_TEXT, Domain
 from rasa.shared.core.events import BotUttered, UserUttered
+from rasa.shared.core.flows.constants import KEY_TRANSLATION
 from rasa.shared.core.trackers import DialogueStateTracker
 from rasa.shared.nlu.constants import (
     KEY_COMPONENT_NAME,
@@ -38,6 +40,7 @@ from rasa.shared.utils.llm import (
     DEFAULT_OPENAI_GENERATE_MODEL_NAME,
     DEFAULT_OPENAI_MAX_GENERATED_TOKENS,
     USER,
+    check_prompt_config_keys_and_warn_if_deprecated,
     combine_custom_and_default_config,
     get_prompt_template,
     llm_factory,
@@ -110,8 +113,15 @@ class ContextualResponseRephraser(
         super().__init__(domain.responses)
 
         self.nlg_endpoint = endpoint_config
+
+        # Warn if the prompt config key is used to set the prompt template
+        check_prompt_config_keys_and_warn_if_deprecated(
+            self.nlg_endpoint.kwargs, "contextual_response_rephraser"
+        )
+
         self.prompt_template = get_prompt_template(
-            self.nlg_endpoint.kwargs.get(PROMPT_CONFIG_KEY),
+            self.nlg_endpoint.kwargs.get(PROMPT_TEMPLATE_CONFIG_KEY)
+            or self.nlg_endpoint.kwargs.get(PROMPT_CONFIG_KEY),
             DEFAULT_RESPONSE_VARIATION_PROMPT_TEMPLATE,
             log_source_component=ContextualResponseRephraser.__name__,
             log_source_method=LOG_COMPONENT_SOURCE_METHOD_INIT,
@@ -215,10 +225,8 @@ class ContextualResponseRephraser(
 
     @measure_llm_latency
     async def _generate_llm_response(self, prompt: str) -> Optional[LLMResponse]:
-        """Use LLM to generate a response.
-
-        Returns an LLMResponse object containing both the generated text
-        (choices) and metadata.
+        """Use LLM to generate a response, returning an LLMResponse object
+        containing both the generated text (choices) and metadata.
 
         Args:
             prompt: The prompt to send to the LLM.
@@ -298,22 +306,23 @@ class ContextualResponseRephraser(
         Returns:
             The response with the rephrased text.
         """
-        if not (response_text := response.get(KEY_RESPONSES_TEXT)):
+        translation_response = response.get(KEY_TRANSLATION) or {}
+        lang_code = getattr(tracker.current_language, "code", None)
+        response_text = translation_response.get(
+            lang_code, response.get(KEY_RESPONSES_TEXT)
+        )
+        if not response_text:
             return response
 
         prompt_template_text = self._template_for_response_rephrasing(response)
-
-        # Last user message (=current input) should always be in prompt if available
         last_message_by_user = getattr(tracker.latest_message, "text", "")
         current_input = (
             f"{USER}: {last_message_by_user}" if last_message_by_user else ""
         )
 
-        # Only summarise conversation history if flagged
         if self.summarize_history:
             history = await self._create_history(tracker)
         else:
-            # Count multiple utterances by bot/user as single turn
             turns_wrapper = (
                 _count_multiple_utterances_as_single_turn
                 if self.count_multiple_utterances_as_single_turn
@@ -356,16 +365,20 @@ class ContextualResponseRephraser(
         )
 
         if not (llm_response and llm_response.choices and llm_response.choices[0]):
-            # If the LLM fails to generate a response, return the original response.
             return response
 
         updated_text = llm_response.choices[0]
+
+        if lang_code in translation_response:
+            response[KEY_TRANSLATION][lang_code] = updated_text
+        else:
+            response[KEY_RESPONSES_TEXT] = updated_text
+
         structlogger.debug(
             "nlg.rewrite.complete",
             response_text=response_text,
             updated_text=updated_text,
         )
-        response[KEY_RESPONSES_TEXT] = updated_text
         return response
 
     def does_response_allow_rephrasing(self, template: Dict[Text, Any]) -> bool:
@@ -399,9 +412,12 @@ class ContextualResponseRephraser(
         Returns:
             The generated response.
         """
-        templated_response = await super().generate(
+        filled_slots = tracker.current_slot_values()
+        stack_context = tracker.stack.current_context()
+        templated_response = self.generate_from_slots(
             utter_action=utter_action,
-            tracker=tracker,
+            filled_slots=filled_slots,
+            stack_context=stack_context,
             output_channel=output_channel,
             **kwargs,
         )
