@@ -249,16 +249,6 @@ class FlowRetrieval(EmbeddingsHealthCheckMixin):
         )
 
         flows_to_embedd = flows.exclude_link_only_flows()
-
-        if not flows_to_embedd:
-            structlogger.debug(
-                "flow_retrieval.populate_vector_store.no_flows_to_embed",
-                event_info=(
-                    "No flows to embed in the vector store, skipping population."
-                ),
-            )
-            return
-
         embeddings = self._create_embedder(self.config)
         documents = self._generate_flow_documents(flows_to_embedd, domain)
         try:
