@@ -1,7 +1,7 @@
 from typing import List, Optional
 
+from rasa.core.available_endpoints import AvailableEndpoints
 from rasa.core.channels import UserMessage
-from rasa.core.utils import AvailableEndpoints
 from rasa.shared.core.trackers import DialogueStateTracker
 from rasa.shared.importers.importer import TrainingDataImporter
 
