@@ -100,7 +100,9 @@ class FlowStep:
         """Most steps allow linking to the next step. But some don't."""
         return True
 
-    def as_json(self) -> Dict[Text, Any]:
+    def as_json(
+        self, step_properties: Optional[Dict[Text, Any]] = None
+    ) -> Dict[Text, Any]:
         """Serialize the FlowStep object.
 
         Returns:
@@ -108,10 +110,13 @@ class FlowStep:
         """
         data: Dict[Text, Any] = {"id": self.id}
 
-        if dumped_next := self.next.as_json():
-            data["next"] = dumped_next
+        if step_properties:
+            data.update(step_properties)
+
         if self.description:
             data["description"] = self.description
+        if dumped_next := self.next.as_json():
+            data["next"] = dumped_next
         if self.metadata:
             data["metadata"] = self.metadata
         return data
@@ -143,6 +148,17 @@ class FlowStep:
         """Return all the utterances used in this step."""
         return set()
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, FlowStep):
+            return False
+
+        return (
+            self.idx == other.idx
+            and self.description == other.description
+            and self.next == other.next
+            and self.flow_id == other.flow_id
+        )
+
 
 @dataclass
 class FlowStepWithFlowReference:
