@@ -38,12 +38,13 @@ from rasa.core.nlg import NaturalLanguageGenerator
 from rasa.core.policies.flows.flow_step_result import FlowActionPrediction
 from rasa.core.policies.policy import Policy, PolicyPrediction
 from rasa.core.processor import MessageProcessor
-from rasa.core.tracker_store import TrackerStore
+from rasa.core.tracker_stores.tracker_store import TrackerStore
 from rasa.dialogue_understanding.commands import Command
 from rasa.dialogue_understanding.generator import (
     CompactLLMCommandGenerator,
     LLMCommandGenerator,
     MultiStepLLMCommandGenerator,
+    SearchReadyLLMCommandGenerator,
     SingleStepLLMCommandGenerator,
 )
 from rasa.dialogue_understanding.generator.flow_retrieval import FlowRetrieval
@@ -66,9 +67,11 @@ from rasa.tracing.instrumentation.intentless_policy_instrumentation import (
 from rasa.tracing.instrumentation.metrics import (
     record_callable_duration_metrics,
     record_compact_llm_command_generator_metrics,
+    record_enterprise_search_policy_metrics,
     record_llm_command_generator_metrics,
     record_multi_step_llm_command_generator_metrics,
     record_request_size_in_bytes,
+    record_search_ready_llm_command_generator_metrics,
     record_single_step_llm_command_generator_metrics,
 )
 from rasa.utils.endpoints import EndpointConfig, concat_url
@@ -291,6 +294,9 @@ SingleStepLLMCommandGeneratorType = TypeVar(
 CompactLLMCommandGeneratorType = TypeVar(
     "CompactLLMCommandGeneratorType", bound=CompactLLMCommandGenerator
 )
+SearchReadyLLMCommandGeneratorType = TypeVar(
+    "SearchReadyLLMCommandGeneratorType", bound=SearchReadyLLMCommandGenerator
+)
 MultiStepLLMCommandGeneratorType = TypeVar(
     "MultiStepLLMCommandGeneratorType", bound=MultiStepLLMCommandGenerator
 )
@@ -326,6 +332,9 @@ def instrument(
     compact_llm_command_generator_class: Optional[
         Type[CompactLLMCommandGeneratorType]
     ] = None,
+    search_ready_llm_command_generator_class: Optional[
+        Type[SearchReadyLLMCommandGeneratorType]
+    ] = None,
     multi_step_llm_command_generator_class: Optional[
         Type[MultiStepLLMCommandGeneratorType]
     ] = None,
@@ -379,6 +388,9 @@ def instrument(
     :param compact_llm_command_generator_class: The `CompactLLMCommandGenerator`
         to be instrumented. If `None` is given, no `CompactLLMCommandGenerator` will
         be instrumented.
+    :param search_ready_llm_command_generator_class: The`SearchReadyLLMCommandGenerator`
+        to be instrumented. If `None` is given, no `SearchReadyLLMCommandGenerator` will
+        be instrumented.
     :param multi_step_llm_command_generator_class: The `MultiStepLLMCommandGenerator`
         to be instrumented. If `None` is given, no `MultiStepLLMCommandGenerator` will
         be instrumented.
@@ -527,6 +539,37 @@ def instrument(
         )
         mark_class_as_instrumented(compact_llm_command_generator_class)
 
+    if (
+        search_ready_llm_command_generator_class is not None
+        and not class_is_instrumented(search_ready_llm_command_generator_class)
+    ):
+        _instrument_method(
+            tracer_provider.get_tracer(
+                search_ready_llm_command_generator_class.__module__
+            ),
+            search_ready_llm_command_generator_class,
+            "invoke_llm",
+            attribute_extractors.extract_attrs_for_llm_based_command_generator,
+            metrics_recorder=record_search_ready_llm_command_generator_metrics,
+        )
+        _instrument_method(
+            tracer_provider.get_tracer(
+                search_ready_llm_command_generator_class.__module__
+            ),
+            search_ready_llm_command_generator_class,
+            "_check_commands_against_startable_flows",
+            attribute_extractors.extract_attrs_for_check_commands_against_startable_flows,
+        )
+        _instrument_perform_health_check_method_for_component(
+            tracer_provider.get_tracer(
+                search_ready_llm_command_generator_class.__module__
+            ),
+            search_ready_llm_command_generator_class,
+            "perform_llm_health_check",
+            attribute_extractors.extract_attrs_for_performing_health_check,
+        )
+        mark_class_as_instrumented(search_ready_llm_command_generator_class)
+
     if multi_step_llm_command_generator_class is not None and not class_is_instrumented(
         multi_step_llm_command_generator_class
     ):
@@ -562,6 +605,7 @@ def instrument(
                 llm_command_generator_class,
                 single_step_llm_command_generator_class,
                 compact_llm_command_generator_class,
+                search_ready_llm_command_generator_class,
                 multi_step_llm_command_generator_class,
             )
         )
@@ -847,8 +891,15 @@ def _instrument_enterprise_search_policy(
     _instrument_method(
         tracer,
         policy_class,
-        "_generate_llm_answer",
-        attribute_extractors.extract_attrs_for_enterprise_search_generate_llm_answer,
+        "_invoke_llm",
+        attribute_extractors.extract_attrs_for_enterprise_search_invoke_llm,
+        metrics_recorder=record_enterprise_search_policy_metrics,
+    )
+    _instrument_method(
+        tracer,
+        policy_class,
+        "_parse_llm_relevancy_check_response",
+        attribute_extractors.extract_attrs_for_enterprise_search_parse_llm_relevancy_check_response,
     )
     _instrument_perform_health_check_method_for_component(
         tracer_provider.get_tracer(policy_class.__module__),
