@@ -1,7 +1,6 @@
 import logging
 import re
 import string
-import sys
 from collections import defaultdict
 from typing import Any, Dict, List, Optional, Set, Text, Tuple
 
@@ -17,6 +16,7 @@ import rasa.shared.utils.io
 from rasa.core.channels import UserMessage
 from rasa.dialogue_understanding.stack.frames import PatternFlowStackFrame
 from rasa.engine.language import Language
+from rasa.exceptions import ValidationError
 from rasa.shared.constants import (
     ASSISTANT_ID_DEFAULT_VALUE,
     ASSISTANT_ID_KEY,
@@ -237,8 +237,7 @@ class Validator:
                     "validator.verify_intents_in_stories_or_flows.not_used",
                     intent=intent,
                     event_info=(
-                        f"The intent '{intent}' is not used "
-                        f"in any story, rule or flow."
+                        f"The intent '{intent}' is not used in any story, rule or flow."
                     ),
                 )
                 everything_is_alright = ignore_warnings or everything_is_alright
@@ -605,7 +604,7 @@ class Validator:
                     "validator.config_missing_mandatory_key",
                     key=key,
                     event_info=(
-                        f"The config file is missing the " f"'{key}' mandatory key."
+                        f"The config file is missing the '{key}' mandatory key."
                     ),
                 )
 
@@ -1014,11 +1013,6 @@ class Validator:
             for sv in slot_value
         ]
         if not all(slot_values_validity):
-            invalid_slot_values = [
-                sv
-                for (sv, slot_value_valid) in zip(slot_value, slot_values_validity)
-                if not slot_value_valid
-            ]
             structlogger.error(
                 "validator.verify_predicates.link.invalid_condition",
                 step=step_id,
@@ -1027,8 +1021,7 @@ class Validator:
                 event_info=(
                     f"Detected invalid condition '{link_condition}' "
                     f"at step '{step_id}' for flow id '{flow_id}'. "
-                    f"Values {invalid_slot_values} are not valid values "
-                    f"for slot {slot_name}. "
+                    f"The condition contains invalid values for slot {slot_name}. "
                     f"Please make sure that all conditions are valid."
                 ),
             )
@@ -1798,7 +1791,10 @@ class Validator:
                 "rasa.validator.verify_studio_supported_validations.empty_domain",
                 event_info="Encountered empty domain during validation.",
             )
-            sys.exit(1)
+            raise ValidationError(
+                code="rasa.validator.verify_studio_supported_validations.empty_domain",
+                event_info="Encountered empty domain during validation.",
+            )
 
         self.warn_if_config_mandatory_keys_are_not_set()
 
@@ -1998,7 +1994,7 @@ class Validator:
                         "validator.validate_conditional_response_variation_predicates.invalid_slot",
                         utter=utter_name,
                         event_info=(
-                            f"Detected invalid slot '{slot_namespace[1]}' in "
+                            f"Detected invalid slot '{slot_namespace[0]}' in "
                             f"condition '{condition}' for response variation "
                             f"'{utter_name}'. Please make sure that all slots "
                             f"are specified in the domain file."
